#!/usr/bin/perl -w
#
# file    : download-stanford-files.pl
# changed : 2005-02-09
# author  : Bas Couwenberg <sebastic@xs4all.nl>
#

use strict;
use LWP::UserAgent;
use File::Basename;
use Date::Calc;
use Getopt::Mixed "nextOption";

my $version = '0.1.2';
my $wwwroot = '/var/www/folding/mod_perl/';

# default configuration
my %cfg = ( 
		self			=> 'download-stanford-files.pl',
		version			=> $version,
		'stanford-data-dir'	=> $wwwroot.'stanford-data/',
		'team-summary-url'	=> 'http://vspx27.stanford.edu/daily_team_summary.txt',
		'user-summary-url'	=> 'http://vspx27.stanford.edu/daily_user_summary.txt',
		'project-summary-url' 	=> 'http://vspx27.stanford.edu/psummaryC.html',
		useragent		=> 'download-stanford-files.pl/'.$version,
		verbose 		=> 0
	);

Getopt::Mixed::init("team-summary-url=s user-summary-url=s project-summary-url=s stanford-data-dir=s useragent=s verbose help");
while (my ($option, $value) = nextOption()) {
	if($option eq 'help') {
		print_usage();
		Getopt::Mixed::cleanup();
		exit;
	}
	
	$cfg{'stanford-data-dir'}   = $value if($option eq 'stanford-data-dir');
	$cfg{'team-summary-url'}    = $value if($option eq 'team-summary-url');
	$cfg{'user-summary-url'}    = $value if($option eq 'user-summary-url');
	$cfg{'project-summary-url'} = $value if($option eq 'project-summary-url');
	$cfg{useragent}             = $value if($option eq 'useragent');
	$cfg{verbose}       	    = 1      if($option eq 'verbose');
}
Getopt::Mixed::cleanup();

# check parameters
if(!$cfg{'team-summary-url'} || $cfg{'team-summary-url'} =~ /^\s*$/ || &invalid_url($cfg{'team-summary-url'})) {
	print "Error: Invalid Team Summary URL! ($cfg{'team-summary-url'})\n";
	exit;
}
elsif(!$cfg{'user-summary-url'} || $cfg{'user-summary-url'} =~ /^\s*$/ || &invalid_url($cfg{'user-summary-url'})) {
	print "Error: Invalid User Summary URL! ($cfg{'user-summary-url'})\n";
	exit;
}
elsif(!$cfg{'project-summary-url'} || $cfg{'project-summary-url'} =~ /^\s*$/ || &invalid_url($cfg{'project-summary-url'})) {
	print "Error: Invalid Project Summary URL! ($cfg{'project-summary-url'})\n";
	exit;
}
elsif(!$cfg{useragent} || $cfg{useragent} eq '' || $cfg{useragent} =~ /^\s+$/) {
	print "Error: Invalid User-Agent! ($cfg{useragent})\n";
	exit;
}
elsif(!$cfg{'stanford-data-dir'} || !-w $cfg{'stanford-data-dir'}) {
	print "Error: Cannot write Stanford Data Dir! ($cfg{'stanford-data-dir'})\n";
	exit;
}
		

my $ua = LWP::UserAgent->new;
   $ua->agent($cfg{useragent});

my @url = ( $cfg{'team-summary-url'}, 
	    $cfg{'user-summary-url'},
	    $cfg{'project-summary-url'} );

foreach my $url (@url) {
	print "\nRetrieving URL: $url\n" if($cfg{verbose});
	my $req = HTTP::Request->new(GET => $url);

	my $res = $ua->request($req);
	if($res->is_success) {
		my $dir  = $cfg{'stanford-data-dir'};
		my $file = $dir.basename($url);

		print "Saving file as: $file\n" if($cfg{verbose});
	
		open(F, "> $file") || die "Cannot open file ($!)";
		print F $res->content;
		close F;

		my ($year,$month,$day,$hour,$min,$sec) = Date::Calc::Today_and_Now();
		
		$month = '0'.$month if(length($month) == 1);
		$day   = '0'.$day   if(length($day) == 1);
		$hour  = '0'.$hour  if(length($hour) == 1);
		$min   = '0'.$min   if(length($min) == 1);
		$sec   = '0'.$sec   if(length($sec) == 1);

		my $update_file = $file.'.lastupdate';
		
		print "Logging update: $update_file\n" if($cfg{verbose});
		
		open(F, "> $update_file") || die "Cannot open file ($!)";
		print F "$year-$month-$day $hour:$min:$sec\n";
		close F;
	}
	else {
		print "Error retrieving $url\n";
	}
}

################[ Subroutines ]################

sub invalid_url {
	my $url = $_[0];
	
	my $ua  = LWP::UserAgent->new();
	   $ua->agent($cfg{useragent});
	
	my $req = HTTP::Request->new(HEAD => $url);
	
	my $res = $ua->request($req);
	if($res->is_success()) {
		return 0;
	}
	elsif($res->is_error()) {
		return 1;
	}
	else {
		return 1;
	}
}

sub print_usage {
	my $verbose = $cfg{verbose} ? 'on' : 'off';

	print "$cfg{self} v$cfg{version}\n\n";
	print "Usage: ".basename($0)." [OPTIONS]\n\n";
	print "Options:\n";
	print "--stanford-data-dir <dir>    Directory to save downloaded files    (default: $cfg{'stanford-data-dir'})\n";
	print "--team-summary-url <url>     URL to Stanfords team summary file    (default: $cfg{'team-summary-url'})\n";
	print "--user-summary-url <url>     URL to Stanfords user summary file    (default: $cfg{'user-summary-url'})\n";
	print "--project-summary-url <url>  URL to Stanfords project summary file (default: $cfg{'project-summary-url'})\n";
	print "--useragent <name>           User-Agent string                     (default: $cfg{'useragent'})\n";
	print "--verbose                    Verbose (debug) output                (default: $verbose)\n";
	print "--help                       Print this information\n";
}
